@@ -25,6 +25,7 @@
 #include "timer.h"
 #include "test.h"
 
+bool warnings_for_timers = true;
 bool debug_timers = false;
 #define MOCK_ORIGIN 1000000
 
@@ -59,7 +60,7 @@ void init_timer(Timer *timer, const char *name)
 
 void destroy_timer(Timer *timer)
 {
-    if (!g_list_find(timers, timer)) {
+    if (warnings_for_timers && !g_list_find(timers, timer)) {
         fprintf(stderr, RED "tint2: Attempt to destroy nonexisting timer: %s" RESET "\n", timer->name_);
         return;
     }
